@@ -3,7 +3,7 @@
  *
  * Sysfs utility to list buses, classes, and devices
  *
- * Copyright (C) IBM Corp. 2003
+ * Copyright (C) IBM Corp. 2003-2005
  *
  *	This program is free software; you can redistribute it and/or modify it
  *	under the terms of the GNU General Public License as published by the
@@ -19,11 +19,13 @@
  *	675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
+
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <mntent.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -33,10 +35,24 @@
 #include "libsysfs.h"
 #include "names.h"
 
+extern char *my_strncpy(char *to, const char *from, size_t max);
+#define safestrcpy(to, from)		my_strncpy(to, from, sizeof(to))
+#define safestrcpymax(to, from, max)	my_strncpy(to, from, max)
+
+extern char *my_strncat(char *to, const char *from, size_t max);
+#define safestrcat(to, from)	my_strncat(to, from, sizeof(to) - strlen(to) - 1)
+
+#define safestrcatmax(to, from, max) \
+do { \
+	to[max-1] = '\0'; \
+	strncat(to, from, max - strlen(to)-1); \
+} while (0)
+
 /* Command Options */
 static int show_options = 0;		/* bitmask of show options */
 static char *attribute_to_show = NULL;	/* show value for this attribute */
 static char *device_to_show = NULL;	/* show only this bus device */
+static char sysfs_mnt_path[SYSFS_PATH_MAX]; /* sysfs mount point */
 struct pci_access *pacc = NULL;
 char *show_bus = NULL;
 
@@ -54,7 +70,7 @@ static void show_class_device(struct sysfs_class_device *dev, int level);
 
 #define SHOW_ALL		0xff
 
-static char cmd_options[] = "aA:b:c:CdDhpPr:v";
+static char cmd_options[] = "aA:b:c:dDhm:pP:v";
 
 /*
  * binary_files - defines existing sysfs binary files. These files will be
@@ -85,12 +101,10 @@ static void usage(void)
 	fprintf(stdout, "\t-c <class_name>\t\tShow a specific class\n");
 	fprintf(stdout, "\t-d\t\t\tShow only devices\n");
 	fprintf(stdout, "\t-h\t\t\tShow usage\n");
+	fprintf(stdout, "\t-m <module_name>\tShow a specific module\n");
 	fprintf(stdout, "\t-p\t\t\tShow path to device/driver\n");
-	fprintf(stdout,
-		"\t-r <root_device>\tShow a specific root device tree\n");
 	fprintf(stdout, "\t-v\t\t\tShow all attributes with values\n");
 	fprintf(stdout, "\t-A <attribute_name>\tShow attribute value\n");
-	fprintf(stdout, "\t-C\t\t\tShow device's children\n");
 	fprintf(stdout, "\t-D\t\t\tShow only drivers\n");
 	fprintf(stdout, "\t-P\t\t\tShow device's parent\n");
 }
@@ -116,40 +130,10 @@ static void remove_end_newline(char *value)
 {
 	char *p = value + (strlen(value) - 1);
 
-	if (p != NULL && *p == '\n')
+	if (p && *p == '\n')
 		*p = '\0';
 }
 
-/**
- * show_device_children: prints out device subdirs.
- * @children: dlist of child devices.
- */
-static void show_device_children(struct sysfs_device *device, int level)
-{
-	struct sysfs_device *temp_device = NULL, *child = NULL;
-	int flag = 1;
-
-	temp_device = sysfs_open_device_tree(device->path);
-	if (temp_device) {
-		if (temp_device->children) {
-			fprintf(stdout, "\n");
-			dlist_for_each_data(temp_device->children, child,
-					struct sysfs_device) {
-				if (strncmp(child->name, "power", 5) == 0)
-					continue;
-				if (flag) {
-					flag--;
-					indent(level);
-					fprintf(stdout, "Device \"%s\"'s children\n",
-							temp_device->name);
-				}
-				show_device(child, (level+2));
-			}
-		}
-		sysfs_close_device_tree(temp_device);
-	}
-}
-
 /**
  * isbinaryvalue: checks to see if attribute is binary or not.
  * @attr: attribute to check.
@@ -159,7 +143,7 @@ static int isbinaryvalue(struct sysfs_attribute *attr)
 {
 	int i;
 
-	if (attr == NULL || attr->value == NULL)
+	if (!attr || !attr->value)
 		return 0;
 
 	for (i = 0; i < binfiles; i++)
@@ -175,11 +159,11 @@ static int isbinaryvalue(struct sysfs_attribute *attr)
  */
 static void show_attribute_value(struct sysfs_attribute *attr, int level)
 {
-	if (attr == NULL)
+	if (!attr)
 		return;
 
 	if (attr->method & SYSFS_METHOD_SHOW) {
-		if (isbinaryvalue(attr) != 0) {
+		if (isbinaryvalue(attr)) {
 			int i;
 			for (i = 0; i < attr->len; i++) {
 				if (!(i % 16) && (i != 0)) {
@@ -192,7 +176,7 @@ static void show_attribute_value(struct sysfs_attribute *attr, int level)
 			}
 			fprintf(stdout, "\n");
 
-		} else if (attr->value != NULL && strlen(attr->value) > 0) {
+		} else if (attr->value && strlen(attr->value) > 0) {
 			remove_end_newline(attr->value);
 			fprintf(stdout, "\"%s\"\n", attr->value);
 		} else
@@ -208,7 +192,7 @@ static void show_attribute_value(struct sysfs_attribute *attr, int level)
  */
 static void show_attribute(struct sysfs_attribute *attr, int level)
 {
-	if (attr == NULL)
+	if (!attr)
 		return;
 
 	if (show_options & SHOW_ALL_ATTRIB_VALUES) {
@@ -235,12 +219,12 @@ static void show_attribute(struct sysfs_attribute *attr, int level)
  */
 static void show_attributes(struct dlist *attributes, int level)
 {
-	if (attributes != NULL) {
-		struct sysfs_attribute *cur = NULL;
+	if (attributes) {
+		struct sysfs_attribute *cur;
 
 		dlist_for_each_data(attributes, cur,
 				struct sysfs_attribute) {
-			show_attribute(cur, (level));
+			show_attribute(cur, level);
 		}
 	}
 }
@@ -251,7 +235,7 @@ static void show_attributes(struct dlist *attributes, int level)
  */
 static void show_device_parent(struct sysfs_device *device, int level)
 {
-	struct sysfs_device *parent = NULL;
+	struct sysfs_device *parent;
 
 	parent = sysfs_get_device_parent(device);
 	if (parent) {
@@ -268,28 +252,34 @@ static void show_device_parent(struct sysfs_device *device, int level)
  */
 static void show_device(struct sysfs_device *device, int level)
 {
-	struct dlist *attributes = NULL;
+	struct dlist *attributes;
         unsigned int vendor_id, device_id;
         char buf[128], value[256], path[SYSFS_PATH_MAX];
 
-	if (device != NULL) {
+	if (device) {
 		indent(level);
-		if (show_bus != NULL && (!(strcmp(show_bus, "pci")))) {
+		if (show_bus && (!(strcmp(show_bus, "pci")))) {
 			fprintf(stdout, "%s ", device->bus_id);
 			memset(path, 0, SYSFS_PATH_MAX);
 			memset(value, 0, SYSFS_PATH_MAX);
 			safestrcpy(path, device->path);
 			safestrcat(path, "/config");
-			if ((sysfs_read_attribute_value(path,
-							value, 256)) == 0) {
-				vendor_id = get_pciconfig_word
-						(PCI_VENDOR_ID, value);
-				device_id = get_pciconfig_word
-						(PCI_DEVICE_ID, value);
-				fprintf(stdout, "%s\n",
-					pci_lookup_name(pacc, buf, 128,
-					PCI_LOOKUP_VENDOR | PCI_LOOKUP_DEVICE,
+			struct sysfs_attribute *attr;
+			attr = sysfs_open_attribute(path);
+			if (attr) {
+				if (!sysfs_read_attribute(attr)) {
+					vendor_id = get_pciconfig_word(PCI_VENDOR_ID,
+						       	(unsigned char *)attr->value);
+					device_id = get_pciconfig_word(PCI_DEVICE_ID,
+						       	(unsigned char *)attr->value);
+					fprintf(stdout, "%s\n",
+						pci_lookup_name(pacc,
+						buf, 128,
+						PCI_LOOKUP_VENDOR |
+						PCI_LOOKUP_DEVICE,
 						vendor_id, device_id, 0, 0));
+				}
+				sysfs_close_attribute(attr);
 			} else
 				fprintf(stdout, "\n");
 		} else
@@ -304,15 +294,11 @@ static void show_device(struct sysfs_device *device, int level)
 		if (show_options & (SHOW_ATTRIBUTES | SHOW_ATTRIBUTE_VALUE |
 					SHOW_ALL_ATTRIB_VALUES)) {
 			attributes = sysfs_get_device_attributes(device);
-			if (attributes != NULL)
+			if (attributes)
 				show_attributes(attributes, (level+2));
 		}
-		if ((device_to_show != NULL) &&
-				(show_options & SHOW_CHILDREN)) {
-			show_options &= ~SHOW_CHILDREN;
-			show_device_children(device, (level+2));
-		}
-		if ((device_to_show != NULL) && (show_options & SHOW_PARENT)) {
+
+		if ((device_to_show) && (show_options & SHOW_PARENT)) {
 			show_options &= ~SHOW_PARENT;
 			show_device_parent(device, (level+2));
 		}
@@ -328,12 +314,12 @@ static void show_device(struct sysfs_device *device, int level)
  */
 static void show_driver_attributes(struct sysfs_driver *driver, int level)
 {
-	if (driver != NULL) {
-		struct dlist *attributes = NULL;
+	if (driver) {
+		struct dlist *attributes;
 
 		attributes = sysfs_get_driver_attributes(driver);
-		if (attributes != NULL) {
-			struct sysfs_attribute *cur = NULL;
+		if (attributes) {
+			struct sysfs_attribute *cur;
 
 			dlist_for_each_data(attributes, cur,
 					struct sysfs_attribute) {
@@ -350,11 +336,11 @@ static void show_driver_attributes(struct sysfs_driver *driver, int level)
  */
 static void show_driver(struct sysfs_driver *driver, int level)
 {
-	struct dlist *devlist = NULL;
+	struct dlist *devlist;
 
-	if (driver != NULL) {
+	if (driver) {
 		indent(level);
-		fprintf (stdout, "Driver = \"%s\"\n", driver->name);
+		fprintf(stdout, "Driver = \"%s\"\n", driver->name);
 		if (show_options & (SHOW_PATH | SHOW_ALL_ATTRIB_VALUES)) {
 			indent(level);
 			fprintf(stdout, "Driver path = \"%s\"\n",
@@ -364,8 +350,8 @@ static void show_driver(struct sysfs_driver *driver, int level)
 			    | SHOW_ALL_ATTRIB_VALUES))
 			show_driver_attributes(driver, (level+2));
 		devlist = sysfs_get_driver_devices(driver);
-		if (devlist != NULL) {
-			struct sysfs_device *cur = NULL;
+		if (devlist) {
+			struct sysfs_device *cur;
 
 			indent(level+2);
 			fprintf(stdout, "Devices using \"%s\" are:\n",
@@ -385,30 +371,6 @@ static void show_driver(struct sysfs_driver *driver, int level)
 	}
 }
 
-/**
- * show_device_tree: prints out device tree.
- * @root: root device
- */
-static void show_device_tree(struct sysfs_device *root, int level)
-{
-	if (root != NULL) {
-		struct sysfs_device *cur = NULL;
-
-		if (device_to_show == NULL || (strcmp(device_to_show,
-			    root->bus_id) == 0)) {
-			show_device(root, level);
-		}
-		if (root->children != NULL) {
-			dlist_for_each_data(root->children, cur,
-					struct sysfs_device) {
-				if (strncmp(cur->name, "power", 5) == 0)
-					continue;
-				show_device_tree(cur, (level+2));
-			}
-		}
-	}
-}
-
 /**
  * show_sysfs_bus: prints out everything on a bus.
  * @busname: bus to print.
@@ -416,13 +378,13 @@ static void show_device_tree(struct sysfs_device *root, int level)
  */
 static int show_sysfs_bus(char *busname)
 {
-	struct sysfs_bus *bus = NULL;
-	struct sysfs_device *curdev = NULL;
-	struct sysfs_driver *curdrv = NULL;
-	struct dlist *devlist = NULL;
-	struct dlist *drvlist = NULL;
+	struct sysfs_bus *bus;
+	struct sysfs_device *curdev;
+	struct sysfs_driver *curdrv;
+	struct dlist *devlist;
+	struct dlist *drvlist;
 
-	if (busname == NULL) {
+	if (!busname) {
 		errno = EINVAL;
 		return 1;
 	}
@@ -437,11 +399,10 @@ static int show_sysfs_bus(char *busname)
 		fprintf(stdout, "\n");
 	if (show_options & SHOW_DEVICES) {
 		devlist = sysfs_get_bus_devices(bus);
-		if (devlist != NULL) {
+		if (devlist) {
 			dlist_for_each_data(devlist, curdev,
 						struct sysfs_device) {
-				if (device_to_show == NULL ||
-						(strcmp(device_to_show,
+				if (!device_to_show || (strcmp(device_to_show,
 							curdev->bus_id) == 0))
 					show_device(curdev, 2);
 			}
@@ -449,7 +410,7 @@ static int show_sysfs_bus(char *busname)
 	}
 	if (show_options & SHOW_DRIVERS) {
 		drvlist = sysfs_get_bus_drivers(bus);
-		if (drvlist != NULL) {
+		if (drvlist) {
 			dlist_for_each_data(drvlist, curdrv,
 					struct sysfs_driver) {
 				show_driver(curdrv, 2);
@@ -466,7 +427,7 @@ static int show_sysfs_bus(char *busname)
  */
 static void show_classdev_parent(struct sysfs_class_device *dev, int level)
 {
-	struct sysfs_class_device *parent = NULL;
+	struct sysfs_class_device *parent;
 
 	parent = sysfs_get_classdev_parent(dev);
 	if (parent) {
@@ -484,11 +445,10 @@ static void show_classdev_parent(struct sysfs_class_device *dev, int level)
  */
 static void show_class_device(struct sysfs_class_device *dev, int level)
 {
-	struct dlist *attributes = NULL;
-	struct sysfs_device *device = NULL;
-	struct sysfs_driver *driver = NULL;
+	struct dlist *attributes;
+	struct sysfs_device *device;
 
-	if (dev != NULL) {
+	if (dev) {
 		indent(level);
 		fprintf(stdout, "Class Device = \"%s\"\n", dev->name);
 		if (show_options & (SHOW_PATH | SHOW_ALL_ATTRIB_VALUES)) {
@@ -499,23 +459,17 @@ static void show_class_device(struct sysfs_class_device *dev, int level)
 		if (show_options & (SHOW_ATTRIBUTES | SHOW_ATTRIBUTE_VALUE
 		    | SHOW_ALL_ATTRIB_VALUES)) {
 			attributes = sysfs_get_classdev_attributes(dev);
-			if (attributes != NULL)
+			if (attributes)
 				show_attributes(attributes, (level+2));
 			fprintf(stdout, "\n");
 		}
 		if (show_options & (SHOW_DEVICES | SHOW_ALL_ATTRIB_VALUES)) {
 			device = sysfs_get_classdev_device(dev);
-			if (device != NULL) {
+			if (device) {
 				show_device(device, (level+2));
 			}
 		}
-		if (show_options & (SHOW_DRIVERS | SHOW_ALL_ATTRIB_VALUES)) {
-			driver = sysfs_get_classdev_driver(dev);
-			if (driver != NULL) {
-				show_driver(driver, (level+2));
-			}
-		}
-		if ((device_to_show != NULL) && (show_options & SHOW_PARENT)) {
+		if ((device_to_show) && (show_options & SHOW_PARENT)) {
 			show_options &= ~SHOW_PARENT;
 			show_classdev_parent(dev, level+2);
 		}
@@ -532,11 +486,11 @@ static void show_class_device(struct sysfs_class_device *dev, int level)
  */
 static int show_sysfs_class(char *classname)
 {
-	struct sysfs_class *cls = NULL;
-	struct sysfs_class_device *cur = NULL;
-	struct dlist *clsdevlist = NULL;
+	struct sysfs_class *cls;
+	struct sysfs_class_device *cur;
+	struct dlist *clsdevlist;
 
-	if (classname == NULL) {
+	if (!classname) {
 		errno = EINVAL;
 		return 1;
 	}
@@ -547,7 +501,7 @@ static int show_sysfs_class(char *classname)
 	}
 	fprintf(stdout, "Class = \"%s\"\n\n", classname);
 	clsdevlist = sysfs_get_class_devices(cls);
-	if (clsdevlist != NULL) {
+	if (clsdevlist) {
 		dlist_for_each_data(clsdevlist, cur,
 				struct sysfs_class_device) {
 			if (device_to_show == NULL || (strcmp(device_to_show,
@@ -560,42 +514,68 @@ static int show_sysfs_class(char *classname)
 	return 0;
 }
 
-/**
- * show_sysfs_root: prints out sysfs root device tree
- * @rootname: device root to print.
- * returns 0 with success and 1 with error.
- */
-static int show_sysfs_root(char *rootname)
+static int show_sysfs_module(char *module)
 {
-	struct sysfs_root_device *root = NULL;
-	struct sysfs_device *device = NULL;
-	struct dlist *devlist = NULL;
+	struct sysfs_module *mod = NULL;
 
-	if (rootname == NULL) {
+	if (!module) {
 		errno = EINVAL;
 		return 1;
 	}
-	root = sysfs_open_root_device(rootname);
-	if (root == NULL) {
-		fprintf(stderr, "Error opening root device %s\n", rootname);
+
+	mod = sysfs_open_module(module);
+	if (mod == NULL) {
+		fprintf(stderr, "Error opening module %s\n", module);
 		return 1;
 	}
+	fprintf(stdout, "Module = \"%s\"\n\n", module);
+	if (show_options & (SHOW_ATTRIBUTES | SHOW_ATTRIBUTE_VALUE
+	    | SHOW_ALL_ATTRIB_VALUES)) {
+		struct dlist *attributes = NULL;
+		struct sysfs_attribute *cur;
 
-	devlist = sysfs_get_root_devices(root);
-	if (devlist != NULL) {
-		fprintf(stdout, "Root Device = \"%s\"\n\n", rootname);
-
-		if (devlist != NULL) {
-			dlist_for_each_data(devlist, device,
-						struct sysfs_device) {
-				if (strncmp(device->name, "power", 5) == 0)
-					continue;
-				show_device_tree(device, 2);
+		attributes = sysfs_get_module_attributes(mod);
+		if (attributes) {
+			if (show_options & (SHOW_ATTRIBUTES
+			    | SHOW_ALL_ATTRIB_VALUES)) {
+				indent(2);
+				fprintf(stdout, "Attributes:\n");
+			}
+			dlist_for_each_data(attributes, cur,
+					struct sysfs_attribute) {
+				show_attribute(cur, (4));
 			}
 		}
+		attributes = sysfs_get_module_parms(mod);
+		if (attributes) {
+			if (show_options & (SHOW_ATTRIBUTES
+			    | SHOW_ALL_ATTRIB_VALUES)) {
+				fprintf(stdout, "\n");
+				indent(2);
+				fprintf(stdout, "Parameters:\n");
+			}
+			dlist_for_each_data(attributes, cur,
+					struct sysfs_attribute) {
+				show_attribute(cur, (4));
+			}
+		}
+		attributes = sysfs_get_module_sections(mod);
+		if (attributes) {
+			if (show_options & (SHOW_ATTRIBUTES
+			    | SHOW_ALL_ATTRIB_VALUES)) {
+				fprintf(stdout, "\n");
+				indent(2);
+				fprintf(stdout, "Sections:\n");
+			}
+			dlist_for_each_data(attributes, cur,
+					struct sysfs_attribute) {
+				show_attribute(cur, (4));
+			}
+			fprintf(stdout, "\n");
+		}
 	}
-	sysfs_close_root_device(root);
 
+	sysfs_close_module(mod);
 	return 0;
 }
 
@@ -607,37 +587,54 @@ static int show_sysfs_root(char *rootname)
 static int show_default_info(void)
 {
 	char subsys[SYSFS_NAME_LEN];
-	struct dlist *list = NULL;
-	char *cur = NULL;
+	struct dlist *list;
+	char *cur;
 	int retval = 0;
 
-	safestrcpy(subsys, SYSFS_BUS_NAME);
-	list = sysfs_open_subsystem_list(subsys);
-	if (list != NULL) {
+	safestrcpy(subsys, sysfs_mnt_path);
+	safestrcat(subsys, "/");
+	safestrcat(subsys, SYSFS_BUS_NAME);
+	list = sysfs_open_directory_list(subsys);
+	if (list) {
 		fprintf(stdout, "Supported sysfs buses:\n");
 		dlist_for_each_data(list, cur, char)
 			fprintf(stdout, "\t%s\n", cur);
 		sysfs_close_list(list);
 	}
 
-	safestrcpy(subsys, SYSFS_CLASS_NAME);
-	list = sysfs_open_subsystem_list(subsys);
-	if (list != NULL) {
+	safestrcpy(subsys, sysfs_mnt_path);
+	safestrcat(subsys, "/");
+	safestrcat(subsys, SYSFS_CLASS_NAME);
+	list = sysfs_open_directory_list(subsys);
+	if (list) {
 		fprintf(stdout, "Supported sysfs classes:\n");
 		dlist_for_each_data(list, cur, char)
 			fprintf(stdout, "\t%s\n", cur);
 		sysfs_close_list(list);
 	}
 
-	safestrcpy(subsys, SYSFS_DEVICES_NAME);
-	list = sysfs_open_subsystem_list(subsys);
-	if (list != NULL) {
+	safestrcpy(subsys, sysfs_mnt_path);
+	safestrcat(subsys, "/");
+	safestrcat(subsys, SYSFS_DEVICES_NAME);
+	list = sysfs_open_directory_list(subsys);
+	if (list) {
 		fprintf(stdout, "Supported sysfs devices:\n");
 		dlist_for_each_data(list, cur, char)
 			fprintf(stdout, "\t%s\n", cur);
 		sysfs_close_list(list);
 	}
 
+	safestrcpy(subsys, sysfs_mnt_path);
+	safestrcat(subsys, "/");
+	safestrcat(subsys, SYSFS_MODULE_NAME);
+	list = sysfs_open_directory_list(subsys);
+	if (list) {
+		fprintf(stdout, "Supported sysfs modules:\n");
+		dlist_for_each_data(list, cur, char)
+			fprintf(stdout, "\t%s\n", cur);
+		sysfs_close_list(list);
+	}
+
 	return retval;
 }
 
@@ -647,9 +644,7 @@ static int show_default_info(void)
  */
 static int check_sysfs_is_mounted(void)
 {
-	char path[SYSFS_PATH_MAX];
-
-	if (sysfs_get_mnt_path(path, SYSFS_PATH_MAX) != 0)
+	if (sysfs_get_mnt_path(sysfs_mnt_path, SYSFS_PATH_MAX) != 0)
 		return 0;
 	return 1;
 }
@@ -657,12 +652,12 @@ static int check_sysfs_is_mounted(void)
 /* MAIN */
 int main(int argc, char *argv[])
 {
-/*	char *show_bus = NULL;*/
 	char *show_class = NULL;
+	char *show_module = NULL;
 	char *show_root = NULL;
 	int retval = 0;
 	int opt;
-        char *pci_id_file = "/usr/local/share/pci.ids";
+	char *pci_id_file = PCI_IDS_PATHNAME;
 
 	while((opt = getopt(argc, argv, cmd_options)) != EOF) {
 		switch(opt) {
@@ -685,9 +680,6 @@ int main(int argc, char *argv[])
 		case 'c':
 			show_class = optarg;
 			break;
-		case 'C':
-			show_options |= SHOW_CHILDREN;
-			break;
 		case 'd':
 			show_options |= SHOW_DEVICES;
 			break;
@@ -698,15 +690,15 @@ int main(int argc, char *argv[])
 			usage();
 			exit(0);
 			break;
+		case 'm':
+			show_module = optarg;
+			/* FALLTHRU */
 		case 'p':
 			show_options |= SHOW_PATH;
 			break;
 		case 'P':
 			show_options |= SHOW_PARENT;
 			break;
-		case 'r':
-			show_root = optarg;
-			break;
 		case 'v':
 			show_options |= SHOW_ALL_ATTRIB_VALUES;
 			break;
@@ -742,12 +734,12 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if ((show_bus == NULL && show_class == NULL && show_root == NULL) &&
+	if ((!show_bus && !show_class && !show_module && !show_root) &&
 			(show_options & (SHOW_ATTRIBUTES |
 				SHOW_ATTRIBUTE_VALUE | SHOW_DEVICES |
 				SHOW_DRIVERS | SHOW_ALL_ATTRIB_VALUES))) {
 		fprintf(stderr,
-			"Please specify a bus, class, or root device\n");
+			"Please specify a bus, class, module, or root device\n");
 		usage();
 		exit(1);
 	}
@@ -755,9 +747,7 @@ int main(int argc, char *argv[])
 	if (!(show_options & (SHOW_DEVICES | SHOW_DRIVERS)))
 		show_options |= SHOW_DEVICES;
 
-	if (show_bus != NULL) {
-	/*	if ((!(strcmp(show_bus, "pci"))) &&
-				(show_options & SHOW_DEVICES)) { */
+	if (show_bus) {
 		if ((!(strcmp(show_bus, "pci"))))  {
 			pacc = (struct pci_access *)
 				calloc(1, sizeof(struct pci_access));
@@ -766,17 +756,16 @@ int main(int argc, char *argv[])
 		}
 		retval = show_sysfs_bus(show_bus);
 	}
-	if (show_class != NULL)
+	if (show_class)
 		retval = show_sysfs_class(show_class);
-	if (show_root != NULL)
-		retval = show_sysfs_root(show_root);
 
-	if (show_bus == NULL && show_class == NULL && show_root == NULL)
+	if (show_module)
+		retval = show_sysfs_module(show_module);
+
+	if (!show_bus && !show_class && !show_module && !show_root)
 		retval = show_default_info();
 
-	if (show_bus != NULL) {
-		/*if ((!(strcmp(show_bus, "pci"))) &&
-				(show_options & SHOW_DEVICES)) { */
+	if (show_bus) {
 		if ((!(strcmp(show_bus, "pci"))))  {
 			pci_free_name_list(pacc);
 			free (pacc);
