@@ -40,9 +40,9 @@
 #define val_drv1_attr_name	"dummy2"
 #define inval_name 		"invalid_name"
 #define inval_path 		"/sys/invalid/path"
-#define FUNC_TABLE_SIZE  	(sizeof(func_table)/sizeof(int))
+#define FUNC_TABLE_SIZE  	(sizeof(func_table) / sizeof(void *))
 
-FILE *my_stdout;
+extern FILE *my_stdout;
 
 #define dbg_print(format, arg...) fprintf(my_stdout, format, ## arg)
 
@@ -75,6 +75,7 @@ extern int test_sysfs_path_is_link(int flag);
 extern int test_sysfs_path_is_file(int flag);
 extern int test_sysfs_get_link(int flag);
 extern int test_sysfs_open_directory_list(int flag);
+extern int test_sysfs_open_link_list(int flag);
 extern int test_sysfs_close_list(int flag);
 extern int test_sysfs_close_attribute(int flag);
 extern int test_sysfs_open_attribute(int flag);
